% modified cross-entropy method (univariate normal p.d.f.)
% based on stochastic approximation Hu 2012

% usage example:
%   [S,H,X]=CEm(1000,500,@flip_sphere,50,0.05);
% 
% N: # of samples
% K: # of iterations
% test_func: @+function name of the test, a function to be maximized
% NN: dimension of the test function
% rho: quantile parameter
% samples: number of samples
% H_opt: optimal value
% x_opt: candidate solution

function [samples, H_opt, x_opt] = CEm(N, K, test_func, NN, rho)
mu0 = 60*rand(1,NN)-30;  % -30 < mu0 < 30
sigma0 = 1000*ones(1,NN);   % the variance vector
mu = mu0;
sigma = sigma0;
Ntotal = 0;
samples=[];
k = 1;
while k <= K
  
    % generate samples from current distribution
    X = ones(N,1)*mu + randn(N,NN)*diag(sqrt(sigma));   % each row is a sample, # col = # sample
    Ntotal = Ntotal + N;
    G = feval(test_func,X);

    % calculate the 1-rho quantile
    [G_sort, I_sort] = sort(G,'descend');   % sorted performances
    X_sort = X(I_sort, :);      % sorted samples
    %clear X; clear G;
    x_opt(k,:)=X_sort(1,:);
    H_opt(k)=G_sort(1);
    
    if isnan(H_opt(k))==1
       H_opt(k)=H_opt(k-1);
       x_opt(k,:) =x_opt(k-1,:);
       break;
    end
    
    if k>2 
        if H_opt(k-1)>H_opt(k)
           H_opt(k)=H_opt(k-1);
        end 
    end
    
%-----calculate weights---
 gm = G_sort(ceil(N*rho));
 G_u=(G>=gm);
 w_k=G_u/sum(G_u);

 alpha_k=5/(k+100)^0.501;
 mu_old=mu;
 mu_new=w_k'*X;   
 mu=alpha_k*mu_new+(1-alpha_k)*mu;
 sigma_new=w_k'*( (X - ones(N,1) * mu).^2 );
 sigma=alpha_k*sigma_new+(1-alpha_k)*(sigma+(mu_old-mu).^2);      
    
 samples(k) = Ntotal; 
 k = k+1;
 clear G G_u w_k mu_k var_k theta1_k theta2_k CX_k grad_k Hes_k Hes_inv_k;
end

for l=k:K
samples(l) = Ntotal;
H_opt(l)=H_opt(k-1);
x_opt(l,:) =x_opt(k-1,:);
Ntotal=Ntotal+N;
end
    