% MRAS method (univariate normal p.d.f.)
% N: sample size
% rho: quantile
% NN: dimension of the problem
% v: smoothing parameter for updated parameters 

function [samples, Scurrent, x_opt] = MRAS(N, K, test_func, NN, rho)
v=0.2;  % smoothing parameter
epsilon=1e-5;
Nmin=5*NN;
r=10^(-4);
mu0 = -30+60*rand(1,NN);  % -30 < mu0 < 30
sigma0 = 1000*ones(1,NN);   % the covariance matrix
mu = mu0;
sigma = sigma0;
Ntotal = 0;
Scurrent=zeros(K,1);

k = 0;
while k < K
    k = k+1;
    a=1;
    b=1; %sample increasing rate
    % generate samples from current distribution
    X=randn(N,NN)*diag(sqrt(sigma))+ones(N,1)*mu;
    Ntotal = Ntotal + N*a;
    G = feval(test_func,X);
    
    % calculate the 1-rho quantile
    [G_sort, I_sort] = sort(G);   % sorted performances
    X_sort = X(I_sort, :);      % sorted samples
    x_opt=X_sort(N);
    Scurrent(k)=G_sort(N);
    clear X; clear G;
    gamma_bar = G_sort(ceil(N*(1-rho)));
    
%  update rho (not used, we use constant rho for fair comparision)
%     if k>=2
%        if gamma_bar < (gamma_bar_last+epsilon/2)
%            if G_sort(N-Nmin+1)>=(gamma_bar_last+epsilon/2)
%                I=find(G_sort >=(gamma_bar_last+epsilon/2));
%                gamma_bar=G_sort(I(1));
%                rho=1-I(1)/N;   
%            clear I; 
%           else 
%                N=ceil(b*N);
%                gamma_bar=gamma_bar_last;
%            end
%        end
%     end
%     gamma_bar_last=gamma_bar;
    
    % update parameters of sampling distribution
    elite = find(G_sort >= gamma_bar);  
    nelite = length(elite);
    X_elite = X_sort(elite,:);  % all elite samples have equal weights
    G_elite=G_sort(elite);
    
    f_k=0.99*mvnpdf(X_elite,mu,sigma)+0.01*mvnpdf(X_elite,mu0,sigma0);
    f_k=f_k/max(f_k)+1e-5;
    
    G_s_temp=((exp(r*G_elite))+10^(-5));
    G_s_temp=G_s_temp/max(G_s_temp);
    G_s=G_s_temp.^(k-1)./f_k;
    w_k=G_s/sum(G_s);
    %independent normal
    mu_new=w_k'*X_elite;
    sigma_new=w_k'*( (X_elite - ones(nelite,1) * mu_new).^2 );
    
    % smooth the parameters
    mu = v*mu_new + (1-v)*mu;
    sigma = v*sigma_new + (1-v)*sigma+1e-10;
    x_opt=X_elite(end,:);
    
    clear G_sort X_sort X_elite elite nelite G_s mu_new sigma_new I_sort; 
    samples(k) = Ntotal;
    
if k>2
if Scurrent(k-1)>Scurrent(k) 
        Scurrent(k)=Scurrent(k-1);
end 
end
end